/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the mozCellML code.
 *
 * The Initial Developer of the Original Code is
 * The Bioengineering Institute, University of Auckland.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrew Miller <ak.miller@auckland.ac.nz>
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 */

#include <nspr.h>
#include "../base/public/_xpidlgen/mozICellML.h"
#include "../base/public/_xpidlgen/mozCellMLIIntegrator.h"
#include "../base/public/_xpidlgen/mozCellMLIPointWatcher.h"
#include <nsString.h>
#include <nsXPCOM.h>
#include <nsCOMPtr.h>
#include <nsILocalFile.h>
#include <nsComponentManagerUtils.h>
#include <nsIEventQueueService.h>
#include <nsWeakReference.h>

PRBool loopingForIntegration;

class TestPointWatcher
  : public mozCellMLIPointWatcher, public nsSupportsWeakReference
{
public:
  TestPointWatcher()
  {
  }

  NS_DECL_ISUPPORTS
  NS_DECL_MOZCELLMLIPOINTWATCHER

private:
  ~TestPointWatcher() {};
};

NS_IMETHODIMP
TestPointWatcher::PointAdded(
                             double x,
                             PRUint32 numvariables,
                             mozCellMLIVariable* variables[],
                             double values[]
                            )
{
  printf("Point at %f\n{\n", x);
  for (PRUint32 i = 0; i < numvariables; i++)
  {
    nsAutoString varname;
    variables[i]->GetName(varname);
    nsCAutoString varcname;
    CopyUTF16toUTF8(varname, varcname);
    printf("  %s = %f\n", varcname.get(), values[i]);
  }
  printf("}\n");
  return NS_OK;
}

NS_IMETHODIMP
TestPointWatcher::Done()
{
  printf("Integration completed successfully\n");
  loopingForIntegration = PR_FALSE;
  return NS_OK;
}

NS_IMETHODIMP
TestPointWatcher::Halted()
{
  printf("Integration halted manually!!!\n");
  loopingForIntegration = PR_FALSE;
  return NS_OK;
}

NS_IMETHODIMP
TestPointWatcher::Failed(const nsACString& error)
{
  nsCAutoString le(error);
  printf("Integration failed: %s\n", le.get());
  loopingForIntegration = PR_FALSE;
  return NS_OK;
}

NS_IMPL_ISUPPORTS2(TestPointWatcher, mozCellMLIPointWatcher, nsISupportsWeakReference);

int
main(int argc, char** argv)
{
  nsresult rv;
  if (argc < 7)
  {
    printf("Usage: IntegrateModel URL startx stopx abserr relerr grid\n");
    return -1;
  }

  const char* home = getenv("XPCOM_HOME");
  if (home == NULL)
    rv = NS_InitXPCOM2(nsnull, nsnull, nsnull);
  else
  {
    nsCOMPtr<nsILocalFile> homeDir;
    nsCAutoString home8(home);
    nsAutoString home16;
    CopyUTF8toUTF16(home8, home16);
    NS_NewLocalFile(home16, PR_TRUE, getter_AddRefs(homeDir));
    rv = NS_InitXPCOM2(nsnull, homeDir, nsnull);
  }

  NS_ASSERTION(NS_SUCCEEDED(rv), "NS_InitXPCOM failed");
  
  nsCAutoString url(argv[1]);
  double startx = PR_strtod(argv[2], nsnull);
  double stopx = PR_strtod(argv[3], nsnull);
  double abserr = PR_strtod(argv[4], nsnull);
  double relerr = PR_strtod(argv[5], nsnull);
  double grid = PR_strtod(argv[6], nsnull);

  // Create the model from the URL...
  nsCOMPtr<mozICellML> model(do_CreateInstance("@physiome.org/mozCellML;1"));
  if (model == nsnull)
  {
    printf("Cannot create a model instance.\n");
    return -1;
  }

  // Read the model...
  nsAutoString url16;
  CopyUTF8toUTF16(url, url16);
  nsCAutoString errorMessage;
  rv = model->ReadDocument(url16, errorMessage);
  if (rv != NS_OK)
  {
    printf("Error loading model: Failure code/%s\n", errorMessage.get());
    return -1;
  }
  if (!errorMessage.Equals(""))
  {
    printf("Error loading model: %s\n",
           errorMessage.get());
    return -1;
  }

  nsAutoString err;
  model->CheckValidity(err);
  if (!err.Equals(NS_LITERAL_STRING("")))
  {
    nsCAutoString cerr;
    CopyUTF16toUTF8(err, cerr);
    printf("Model is invalid: %s\n", cerr.get());
  }

  nsCOMPtr<nsILocalFile> lf = do_CreateInstance("@mozilla.org/file/local;1");
  lf->SetLeafName(NS_LITERAL_STRING("integrator.c"));
  lf->CreateUnique(
                   nsIFile::NORMAL_FILE_TYPE,
                   0600
                  );
  nsAutoString cfile;
  lf->GetPath(cfile);

  nsCOMPtr<nsILocalFile> lfx = do_CreateInstance("@mozilla.org/file/local;1");
  lfx->SetLeafName(NS_LITERAL_STRING("integrator"));
  lfx->CreateUnique(
                    nsIFile::NORMAL_FILE_TYPE,
                    0700
                   );
  nsAutoString xfile;
  lfx->GetPath(xfile);

  nsCAutoString xfile8, cfile8;
  CopyUTF16toUTF8(xfile, xfile8);
  CopyUTF16toUTF8(cfile, cfile8);

  nsCAutoString compiler("/usr/bin/gcc -L/home/amil082/mozilla/dist/bin "
                         "-I/home/amil082/mozilla/dist/include/nspr "
                         "-I/home/amil082/usr/lib/mozilla-1.8a2/components "
                         "-lnspr4 -lm -Wall -ggdb ");
  compiler += cfile8 + NS_LITERAL_CSTRING(" -o ") + xfile8;

  loopingForIntegration = PR_TRUE;

  nsCOMPtr<nsIEventQueueService> eqs =
    do_GetService("@mozilla.org/event-queue-service;1");
  nsCOMPtr<nsIEventQueue> eq;
  eqs->PushThreadEventQueue(getter_AddRefs(eq));

  if (eq == nsnull)
  {
    printf("Can't get an event queue.\n");
    return -1;
  }

  nsCOMPtr<mozCellMLIIntegrator> intg;
  rv = model->NewIntegrator(cfile8, compiler, xfile8, getter_AddRefs(intg));
  lf->Remove(PR_FALSE);

  if (rv != NS_OK)
  {
    printf("Integration failed.\n");
    lfx->Remove(PR_FALSE);
    eqs->PopThreadEventQueue(eq);
    return -1;
  }

  TestPointWatcher* pwp = new TestPointWatcher();
  NS_ADDREF(pwp);
  // nsCOMPtr<mozCellMLIPointWatcher> pw = pwp;
  intg->RegisterWatcher(pwp);
  intg->Start(model, startx, stopx, grid, abserr, relerr);

  while (loopingForIntegration)
  {
    PLEvent* event;
    eq->WaitForEvent(&event);
    if (event != NULL)
      eq->HandleEvent(event);
  }

  eqs->PopThreadEventQueue(eq);
  NS_RELEASE(pwp);

  lfx->Remove(PR_FALSE);

  return 0;
}
